@@ -124,8 +124,7 @@ var dataLDRTable =  new JU.Lst ();
 if (isHeaderOnly) spectrum.setHeaderTable (dataLDRTable);
 while (!this.done && (label = this.t.getLabel ()) != null && (value = this.getValue (label)) != null) {
 if (this.isTabularData) {
-this.setTabularDataType (spectrum, label);
-if (!this.processTabularData (spectrum, dataLDRTable, isHeaderOnly)) throw  new JSV.exception.JSVException ("Unable to read JDX file");
+this.processTabularData (spectrum, dataLDRTable, label, isHeaderOnly);
 this.addSpectrum (spectrum, false);
 if (this.isSimulation && spectrum.getXUnits ().equals ("PPM")) spectrum.setHZtoPPM (true);
 spectrum = null;
@@ -145,9 +144,7 @@ continue;
 }}if (label.equals ("##JCAMPDX")) {
 this.setVenderSpecificValues (this.t.rawLine);
 }if (spectrum == null) spectrum =  new JSV.common.Spectrum ();
-if (this.readDataLabel (spectrum, label, value, this.errorLog, this.obscure, isHeaderOnly) && !isHeaderOnly) continue;
-JSV.source.JDXReader.addHeader (dataLDRTable, this.t.rawLabel, value);
-if (!isHeaderOnly && this.checkCustomTags (spectrum, label, value)) continue;
+this.processLabel (spectrum, dataLDRTable, label, value, isHeaderOnly);
 }
 if (isHeaderOnly && spectrum != null) this.addSpectrum (spectrum, false);
 }
@@ -155,6 +152,12 @@ if (!isOK) throw  new JSV.exception.JSVException ("##TITLE record not found");
 this.source.setErrorLog (this.errorLog.toString ());
 return this.source;
 }, "~O,~B");
+Clazz.defineMethod (c$, "processLabel", 
+ function (spectrum, dataLDRTable, label, value, isHeaderOnly) {
+if (!this.readDataLabel (spectrum, label, value, this.errorLog, this.obscure, isHeaderOnly) && !isHeaderOnly) return;
+JSV.source.JDXReader.addHeader (dataLDRTable, this.t.rawLabel, value);
+if (!isHeaderOnly) this.checkCustomTags (spectrum, label, value);
+}, "JSV.common.Spectrum,JU.Lst,~S,~S,~B");
 Clazz.defineMethod (c$, "logError", 
  function (err) {
 this.errorLog.append (this.filePath == null || this.filePath.equals (this.lastErrPath) ? "" : this.filePath).append ("\n").append (err).append ("\n");
@@ -197,8 +200,8 @@ throw e;
 }
 }
 }if (this.acdMolFile != null) JSV.common.JSVFileManager.cachePut ("mol", this.acdMolFile);
-}if (!Float.isNaN (this.nmrMaxY)) spectrum.doNormalize (this.nmrMaxY);
- else if (spectrum.getMaxY () >= 10000) spectrum.doNormalize (1000);
+}if (!Float.isNaN (this.nmrMaxY)) spectrum.normalizeSimulation (this.nmrMaxY);
+ else if (spectrum.getMaxY () >= 10000) spectrum.normalizeSimulation (1000);
 if (this.isSimulation) spectrum.setSimulated (this.filePath);
 this.nSpec++;
 if (this.firstSpec > 0 && this.nSpec < this.firstSpec) return true;
@@ -238,9 +241,7 @@ JU.Logger.debug ("##END= " + this.t.getValue ());
 break;
 }label = tmp;
 if (this.isTabularData) {
-this.setTabularDataType (spectrum, label);
-if (!this.processTabularData (spectrum, dataLDRTable, false)) throw  new JSV.exception.JSVException ("Unable to read Block Source");
-continue;
+this.processTabularData (spectrum, dataLDRTable, label, false);
 }if (label.equals ("##DATATYPE")) {
 if (value.toUpperCase ().equals ("LINK")) {
 this.getBlockSpectra (dataLDRTable);
@@ -267,9 +268,7 @@ if (spectrum.getXYCoords ().length > 0 && !this.addSpectrum (spectrum, forceSub)
 spectrum =  new JSV.common.Spectrum ();
 dataLDRTable =  new JU.Lst ();
 continue;
-}if (this.readDataLabel (spectrum, label, value, this.errorLog, this.obscure, false)) continue;
-JSV.source.JDXReader.addHeader (dataLDRTable, this.t.rawLabel, value);
-if (this.checkCustomTags (spectrum, label, value)) continue;
+}this.processLabel (spectrum, dataLDRTable, label, value, false);
 }
 } catch (e) {
 if (Clazz.exceptionOf (e, Exception)) {
@@ -293,7 +292,7 @@ Clazz.defineMethod (c$, "getNTupleSpectra",
 var minMaxY =  Clazz.newDoubleArray (-1, [1.7976931348623157E308, 4.9E-324]);
 this.blockID = Math.random ();
 var isOK = true;
-if (this.firstSpec > 0) spectrum0.numDim = 1;
+if (this.firstSpec > 0) spectrum0.setNumDim (1);
 var isVARNAME = label.equals ("##VARNAME");
 if (!isVARNAME) {
 label = "";
@@ -384,114 +383,110 @@ return this.source;
 }, "JU.Lst,JSV.source.JDXDataObject,~S");
 Clazz.defineMethod (c$, "readDataLabel", 
  function (spectrum, label, value, errorLog, obscure, isHeaderOnly) {
-if (JSV.source.JDXReader.readHeaderLabel (spectrum, label, value, errorLog, obscure)) return true;
+if (!JSV.source.JDXReader.readHeaderLabel (spectrum, label, value, errorLog, obscure)) return false;
 label += " ";
-if (("##MINX ##MINY ##MAXX ##MAXY ##FIRSTY ##DELTAX ##DATACLASS ").indexOf (label) >= 0) return true;
+if (("##MINX ##MINY ##MAXX ##MAXY ##FIRSTY ##DELTAX ##DATACLASS ").indexOf (label) >= 0) return false;
 switch (("##FIRSTX  ##LASTX   ##NPOINTS ##XFACTOR ##YFACTOR ##XUNITS  ##YUNITS  ##XLABEL  ##YLABEL  ##NUMDIM  ##OFFSET  ").indexOf (label)) {
 case 0:
 spectrum.fileFirstX = this.parseAFFN (label, value);
-return true;
+return false;
 case 10:
 spectrum.fileLastX = this.parseAFFN (label, value);
-return true;
+return false;
 case 20:
-spectrum.nPointsFile = Integer.parseInt (value);
-return true;
+spectrum.fileNPoints = Integer.parseInt (value);
+return false;
 case 30:
-spectrum.xFactor = this.parseAFFN (label, value);
-return true;
+spectrum.setXFactor (this.parseAFFN (label, value));
+return false;
 case 40:
 spectrum.yFactor = this.parseAFFN (label, value);
-return true;
+return false;
 case 50:
 spectrum.setXUnits (value);
-return true;
+return false;
 case 60:
 spectrum.setYUnits (value);
-return true;
+return false;
 case 70:
 spectrum.setXLabel (value);
-return false;
+return true;
 case 80:
 spectrum.setYLabel (value);
-return false;
-case 90:
-spectrum.numDim = Integer.parseInt (value);
 return true;
+case 90:
+spectrum.setNumDim (Integer.parseInt (value));
+return false;
 case 100:
-if (spectrum.shiftRefType != 0) {
-if (spectrum.offset == 1.7976931348623157E308) spectrum.offset = this.parseAFFN (label, value);
-spectrum.dataPointNum = 1;
-spectrum.shiftRefType = 1;
+if (!spectrum.isShiftTypeSpecified ()) {
+spectrum.setShiftReference (this.parseAFFN (label, value), 1, 1);
 }return false;
 default:
-if (label.length < 17) return false;
+if (label.length < 17) return true;
 if (label.equals ("##.OBSERVEFREQUENCY ")) {
-spectrum.observedFreq = this.parseAFFN (label, value);
-return true;
+spectrum.setObservedFreq (this.parseAFFN (label, value));
+return false;
 }if (label.equals ("##.OBSERVENUCLEUS ")) {
 spectrum.setObservedNucleus (value);
-return true;
-}if ((label.equals ("##$REFERENCEPOINT ")) && (spectrum.shiftRefType != 0)) {
-spectrum.offset = this.parseAFFN (label, value);
-spectrum.dataPointNum = 1;
-spectrum.shiftRefType = 2;
+return false;
+}if (label.equals ("##$REFERENCEPOINT ") && !spectrum.isShiftTypeSpecified ()) {
+var pt = value.indexOf (" ");
+if (pt > 0) value = value.substring (0, pt);
+spectrum.setShiftReference (this.parseAFFN (label, value), 1, 2);
 return false;
 }if (label.equals ("##.SHIFTREFERENCE ")) {
-if (this.ignoreShiftReference || !(spectrum.dataType.toUpperCase ().contains ("SPECTRUM"))) return true;
+if (this.ignoreShiftReference || !(spectrum.dataType.toUpperCase ().contains ("SPECTRUM"))) return false;
 value = JU.PT.replaceAllCharacters (value, ")(", "");
 var srt =  new java.util.StringTokenizer (value, ",");
-if (srt.countTokens () != 4) return true;
+if (srt.countTokens () != 4) return false;
 try {
 srt.nextToken ();
 srt.nextToken ();
-spectrum.dataPointNum = Integer.parseInt (srt.nextToken ().trim ());
-spectrum.offset = this.parseAFFN (label, srt.nextToken ().trim ());
+var pt = Integer.parseInt (srt.nextToken ().trim ());
+var shift = this.parseAFFN (label, srt.nextToken ().trim ());
+spectrum.setShiftReference (shift, pt, 0);
 } catch (e) {
 if (Clazz.exceptionOf (e, Exception)) {
-return true;
 } else {
 throw e;
 }
 }
-if (spectrum.dataPointNum <= 0) spectrum.dataPointNum = 1;
-spectrum.shiftRefType = 0;
-return true;
-}}
 return false;
+}return true;
+}
 }, "JSV.source.JDXDataObject,~S,~S,JU.SB,~B,~B");
 c$.readHeaderLabel = Clazz.defineMethod (c$, "readHeaderLabel", 
  function (jdxHeader, label, value, errorLog, obscure) {
 switch (("##TITLE#####JCAMPDX###ORIGIN####OWNER#####DATATYPE##LONGDATE##DATE######TIME####").indexOf (label + "#")) {
 case 0:
 jdxHeader.setTitle (obscure || value == null || value.equals ("") ? "Unknown" : value);
-return true;
+return false;
 case 10:
 jdxHeader.jcampdx = value;
 var version = JU.PT.parseFloat (value);
 if (version >= 6.0 || Float.isNaN (version)) {
 if (errorLog != null) errorLog.append ("Warning: JCAMP-DX version may not be fully supported: " + value);
-}return true;
+}return false;
 case 20:
 jdxHeader.origin = (value != null && !value.equals ("") ? value : "Unknown");
-return true;
+return false;
 case 30:
 jdxHeader.owner = (value != null && !value.equals ("") ? value : "Unknown");
-return true;
+return false;
 case 40:
 jdxHeader.dataType = value;
-return true;
+return false;
 case 50:
 jdxHeader.longDate = value;
-return true;
+return false;
 case 60:
 jdxHeader.date = value;
-return true;
+return false;
 case 70:
 jdxHeader.time = value;
-return true;
-}
 return false;
+}
+return true;
 }, "JSV.source.JDXHeader,~S,~S,JU.SB,~B");
 Clazz.defineMethod (c$, "setTabularDataType", 
  function (spectrum, label) {
@@ -501,12 +496,13 @@ if (label.equals ("##PEAKASSIGNMENTS")) spectrum.setDataClass ("PEAKASSIGNMENTS"
  else if (label.equals ("##XYPOINTS")) spectrum.setDataClass ("XYPOINTS");
 }, "JSV.source.JDXDataObject,~S");
 Clazz.defineMethod (c$, "processTabularData", 
- function (spec, table, isHeaderOnly) {
+ function (spec, table, label, isHeaderOnly) {
+this.setTabularDataType (spec, label);
 spec.setHeaderTable (table);
 if (spec.dataClass.equals ("XYDATA")) {
-spec.checkRequiredTokens ();
-if (!isHeaderOnly) this.decompressData (spec, null, false);
-return true;
+spec.checkJDXRequiredTokens ();
+if (!isHeaderOnly) this.decompressData (spec, null);
+return;
 }if (spec.dataClass.equals ("PEAKTABLE") || spec.dataClass.equals ("XYPOINTS")) {
 spec.setContinuous (spec.dataClass.equals ("XYPOINTS"));
 try {
@@ -526,9 +522,9 @@ xyCoords = JSV.common.Coordinate.parseDSV (this.t.getValue (), 1, 1);
 }spec.setXYCoords (xyCoords);
 var fileDeltaX = JSV.common.Coordinate.deltaX (xyCoords[xyCoords.length - 1].getXVal (), xyCoords[0].getXVal (), xyCoords.length);
 spec.setIncreasing (fileDeltaX > 0);
-return true;
-}return false;
-}, "JSV.source.JDXDataObject,JU.Lst,~B");
+return;
+}throw  new JSV.exception.JSVException ("Unable to read JDX file tabular data for line " + this.t.labelLineNo);
+}, "JSV.source.JDXDataObject,JU.Lst,~S,~B");
 Clazz.defineMethod (c$, "readNTUPLECoords", 
  function (spec, nTupleTable, plotSymbols, minMaxY) {
 var list;
@@ -538,16 +534,16 @@ list = nTupleTable.get ("##SYMBOL");
 var index1 = list.indexOf (plotSymbols[0]);
 var index2 = list.indexOf (plotSymbols[1]);
 list = nTupleTable.get ("##VARNAME");
-spec.varName = list.get (index2).toUpperCase ();
+spec.setVarName (list.get (index2).toUpperCase ());
 list = nTupleTable.get (label = "##FACTOR");
-spec.xFactor = this.parseAFFN (label, list.get (index1));
-spec.yFactor = this.parseAFFN (label, list.get (index2));
+spec.setXFactor (this.parseAFFN (label, list.get (index1)));
+spec.setYFactor (this.parseAFFN (label, list.get (index2)));
 list = nTupleTable.get (label = "##LAST");
 spec.fileLastX = this.parseAFFN (label, list.get (index1));
 list = nTupleTable.get (label = "##FIRST");
 spec.fileFirstX = this.parseAFFN (label, list.get (index1));
 list = nTupleTable.get ("##VARDIM");
-spec.nPointsFile = Integer.parseInt (list.get (index1));
+spec.fileNPoints = Integer.parseInt (list.get (index1));
 list = nTupleTable.get ("##UNITS");
 spec.setXUnits (list.get (index1));
 spec.setYUnits (list.get (index2));
@@ -556,7 +552,7 @@ spec.setNucleusAndFreq (list.get (0), false);
 spec.setNucleusAndFreq (list.get (index1), true);
 } else {
 if (spec.nucleusX == null) spec.nucleusX = "?";
-}this.decompressData (spec, minMaxY, true);
+}this.decompressData (spec, minMaxY);
 return true;
 }if (spec.dataClass.equals ("PEAKTABLE") || spec.dataClass.equals ("XYPOINTS")) {
 spec.setContinuous (spec.dataClass.equals ("XYPOINTS"));
@@ -591,14 +587,13 @@ val = val.substring (0, pt);
 }return Double.parseDouble (val);
 }, "~S,~S");
 Clazz.defineMethod (c$, "decompressData", 
- function (spec, minMaxY, isNTUPLE) {
+ function (spec, minMaxY) {
 var errPt = this.errorLog.length ();
 spec.setIncreasing (spec.fileLastX > spec.fileFirstX);
 spec.setContinuous (true);
-var decompressor =  new JSV.source.JDXDecompressor (this.t, spec.fileFirstX, spec.xFactor, spec.yFactor, spec.fileLastX, spec.nPointsFile, isNTUPLE);
-var firstLastX =  Clazz.newDoubleArray (2, 0);
+var decompressor =  new JSV.source.JDXDecompressor (this.t, spec.fileFirstX, spec.fileLastX, spec.xFactor, spec.yFactor, spec.fileNPoints);
 var t = System.currentTimeMillis ();
-var xyCoords = decompressor.decompressData (this.errorLog, firstLastX);
+var xyCoords = decompressor.decompressData (this.errorLog);
 if (JU.Logger.debugging) JU.Logger.debug ("decompression time = " + (System.currentTimeMillis () - t) + " ms");
 spec.setXYCoords (xyCoords);
 var d = decompressor.getMinY ();
@@ -606,25 +601,18 @@ if (minMaxY != null) {
 if (d < minMaxY[0]) minMaxY[0] = d;
 d = decompressor.getMaxY ();
 if (d > minMaxY[1]) minMaxY[1] = d;
-}var freq = (Double.isNaN (spec.freq2dX) ? spec.observedFreq : spec.freq2dX);
-var isHz = freq != 1.7976931348623157E308 && spec.getXUnits ().toUpperCase ().equals ("HZ");
-if (spec.offset != 1.7976931348623157E308 && freq != 1.7976931348623157E308 && spec.dataType.toUpperCase ().contains ("SPECTRUM") && spec.jcampdx.indexOf ("JEOL") < 0) {
-JSV.common.Coordinate.applyShiftReference (xyCoords, spec.dataPointNum, spec.fileFirstX, spec.fileLastX, spec.offset, isHz ? freq : 1, spec.shiftRefType);
-}if (isHz) {
-JSV.common.Coordinate.applyScale (xyCoords, (1.0 / freq), 1);
-spec.setXUnits ("PPM");
-spec.setHZtoPPM (true);
-}if (this.errorLog.length () != errPt) {
-var fileDeltaX = JSV.common.Coordinate.deltaX (spec.fileLastX, spec.fileFirstX, spec.nPointsFile);
+}spec.finalizeCoordinates ();
+if (this.errorLog.length () != errPt) {
+var fileDeltaX = JSV.common.Coordinate.deltaX (spec.fileLastX, spec.fileFirstX, spec.fileNPoints);
 this.logError (spec.getTitle ());
-this.logError ("firstX: " + spec.fileFirstX + " Found " + firstLastX[0]);
-this.logError ("lastX from Header " + spec.fileLastX + " Found " + firstLastX[1]);
+this.logError ("firstX from Header " + spec.fileFirstX);
+this.logError ("lastX from Header " + spec.fileLastX + " Found " + decompressor.lastX);
 this.logError ("deltaX from Header " + fileDeltaX);
-this.logError ("Number of points in Header " + spec.nPointsFile + " Found " + decompressor.getNPointsFound ());
+this.logError ("Number of points in Header " + spec.fileNPoints + " Found " + decompressor.getNPointsFound ());
 } else {
 }if (JU.Logger.debugging) {
 System.err.println (this.errorLog.toString ());
-}}, "JSV.source.JDXDataObject,~A,~B");
+}}, "JSV.source.JDXDataObject,~A");
 c$.addHeader = Clazz.defineMethod (c$, "addHeader", 
 function (table, label, value) {
 var entry;
@@ -660,7 +648,7 @@ break;
 case 40:
 case 50:
 case 60:
-this.acdAssignments = this.mpr.readACDAssignments (spectrum.nPointsFile, pt == 40);
+this.acdAssignments = this.mpr.readACDAssignments ((spectrum).fileNPoints, pt == 40);
 break;
 }
 } catch (e) {
@@ -683,9 +671,9 @@ function () {
 return this.reader.readLine ();
 });
 Clazz.overrideMethod (c$, "setSpectrumPeaks", 
-function (nH, piUnitsX, piUnitsY) {
-this.modelSpectrum.setPeakList (this.peakData, piUnitsX, piUnitsY);
-if (this.modelSpectrum.isNMR ()) this.modelSpectrum.setNHydrogens (nH);
+function (nH, peakXLabel, peakYlabel) {
+this.modelSpectrum.setPeakList (this.peakData, peakXLabel, peakYlabel);
+if (this.modelSpectrum.isNMR ()) this.modelSpectrum.setHydrogenCount (nH);
 }, "~N,~S,~S");
 Clazz.overrideMethod (c$, "addPeakData", 
 function (info) {
