@@ -730,7 +730,7 @@ break;
 case 33:
 if (!this.vwr.ms.haveUnitCells) return "";
 var st = s = this.getFontLineShapeState (shape);
-var iAtom = this.vwr.am.cai;
+var iAtom = this.vwr.am.getUnitCellAtomIndex ();
 if (iAtom >= 0) s += "  unitcell ({" + iAtom + "});\n";
 var uc = this.vwr.getCurrentUnitCell ();
 if (uc != null) {
